/*******************************************************************
* Lionlipobattery.c
*     Li-Ion  Li-Po 
* :16.03.2022 20:48:08
* :  .
********************************************************************/

// 
#include <avr/io.h>
#include <util/delay_basic.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include "Main.h"
#include "Lionlipobattery.h"
#include "Operatingmode.h"
#include "Systemcontrol.h"

bit_lpi indx_lpi={0,0}; //  
var_lpi alter_lpi={{0,0},{0,0},{0,0},{0,0,0,0},{0,0,0,0}}; //  

PROGMEM const byte Lpi_DischargeBattery[2][4]={{2,33,64,0}, //  : I<=0,5 U>3300  <40*
											{5,0,0,100}}; // : I=0,2C U>=3000   I<0.01C  10   
PROGMEM const byte Lpi_ChargeBattery[2][4]={{2,0,64,0}, //  : I<=0,5C U<=4190  <40*C
											{4,42,0,100}}; // : I<=0,25C U<=4200   I<0,01C  10    

EEMEM static byte Lpo_CountcycleAlgorithm[2]={0,0}; //     Li-Po 
EEMEM static byte Lpo_CurrentStatusprocess[2]={0,0}; //     Li-Po  
EEMEM static byte Lin_CountcycleAlgorithm[2]={0,0}; //     Li-Ion 
EEMEM static byte Lin_CurrentStatusprocess[2]={0,0}; //     Li-Ion 

word LPI_MainMode(bit indxnumberbattery_prm,bit indxchargedischarge_prm,word measuredcurrentbattery_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm,word measuredchargebattery_prm,word settcapacitybattery_prm,word settingcurrentbattery_prm,word measuredtemperature_prm) //    Li-Po  Li-Ion 
	{
	word settingcurrentbattery_tmp=0; // 
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: if(indx_lpi.updatingfirstdata==FALSE) //     1  
						{
						indx_lpi.updatingfirstdata=TRUE; //     
						LPI_DefiningParameters(FALSE,indxchargedischarge_prm,settcapacitybattery_prm); //    1
						}
					settingcurrentbattery_tmp=LPI_DetermCurrentcontrol(FALSE,indxchargedischarge_prm,measuredcurrentbattery_prm,measuredchargebattery_prm,settingcurrentbattery_prm,measuredvoltagebattery_prm,measvoltageloadedbattery_prm,measuredtemperature_prm); //      1
					break;
		case TRUE: if(indx_lpi.updatingseconddata==FALSE) //     2 
						{
						indx_lpi.updatingseconddata=TRUE; //    
						LPI_DefiningParameters(TRUE,indxchargedischarge_prm,settcapacitybattery_prm); //    2
						}
					settingcurrentbattery_tmp=LPI_DetermCurrentcontrol(TRUE,indxchargedischarge_prm,measuredcurrentbattery_prm,measuredchargebattery_prm,settingcurrentbattery_prm,measuredvoltagebattery_prm,measvoltageloadedbattery_prm,measuredtemperature_prm); //      2
		}
	return settingcurrentbattery_tmp; //    
	}

void LPI_DefiningParameters(bit indxnumberbattery_prm,bit indxchargedischarge_prm,word settcapacitybattery_prm) //    Ni-Mh 
	{
	switch(indxchargedischarge_prm) //  /
		{
		case FALSE: LPI_ChargeParameters(indxnumberbattery_prm,settcapacitybattery_prm); //  
					break;
		case TRUE: LPI_DischargeParameters(indxnumberbattery_prm,settcapacitybattery_prm); //  
		}
	}

word LPI_DetermCurrentcontrol(bit indxnumberbattery_prm,bit indxchargedischarge_prm,word measuredcurrentbattery_prm,word measuredchargebattery_prm,word settingcurrentbattery_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm,word measuredtemperature_prm) //      
	{
	word settingcurrentbattery_tmp=0; // 
	switch(indxchargedischarge_prm) //  /
		{
		case FALSE: settingcurrentbattery_tmp=LPI_ChargeController(indxnumberbattery_prm,measuredcurrentbattery_prm,measuredchargebattery_prm,settingcurrentbattery_prm,measuredvoltagebattery_prm,measvoltageloadedbattery_prm,measuredtemperature_prm); //  
					break;
		case TRUE: settingcurrentbattery_tmp=LPI_DischargeController(indxnumberbattery_prm,measuredcurrentbattery_prm,measuredvoltagebattery_prm,measvoltageloadedbattery_prm,measuredchargebattery_prm,settingcurrentbattery_prm,measuredtemperature_prm); //  
		}
	return settingcurrentbattery_tmp; //    /
	}

void LPI_ResetVariables(bit indxnumberbattery_prm) //  
	{
	byte counterindxarray_tmp=0; // 
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: while(counterindxarray_tmp<4) //     1 
						{
						alter_lpi.limitfirstparameters[counterindxarray_tmp]=0; //  
						counterindxarray_tmp++; //   
						}
					if(indx_lpi.updatingfirstdata==TRUE) indx_lpi.updatingfirstdata=FALSE; //     
					break;
		case TRUE: while(counterindxarray_tmp<4) //     2 
						{
						alter_lpi.limitsecondparameters[counterindxarray_tmp]=0; //   
						counterindxarray_tmp++; //   
						}
					if(indx_lpi.updatingseconddata==TRUE) indx_lpi.updatingseconddata=FALSE; //     
		}
	alter_lpi.countergeneralusage[indxnumberbattery_prm]=0; //   
	alter_lpi.countercyclesalgorithm[indxnumberbattery_prm]=0; //   / 
	alter_lpi.currentstatusprocess[indxnumberbattery_prm]=0; //  
	}

void LPI_WritelipoDeferred(bit indxnumberbattery_prm) //     Li-Po 
	{
	eeprom_write_byte(&Lpo_CountcycleAlgorithm[indxnumberbattery_prm],alter_lpi.countercyclesalgorithm[indxnumberbattery_prm]); //     
	eeprom_write_byte(&Lpo_CurrentStatusprocess[indxnumberbattery_prm],alter_lpi.currentstatusprocess[indxnumberbattery_prm]); //     
	}

void LPI_WritelionDeferred(bit indxnumberbattery_prm) //     Li-Ion 
	{
	eeprom_write_byte(&Lin_CountcycleAlgorithm[indxnumberbattery_prm],alter_lpi.countercyclesalgorithm[indxnumberbattery_prm]); //     
	eeprom_write_byte(&Lin_CurrentStatusprocess[indxnumberbattery_prm],alter_lpi.currentstatusprocess[indxnumberbattery_prm]); //    
	}

void LPI_ReadlipoDeferred(bit indxnumberbattery_prm) //     Li-Po 
	{
	alter_lpi.countercyclesalgorithm[indxnumberbattery_prm]=eeprom_read_byte(&Lpo_CountcycleAlgorithm[indxnumberbattery_prm]); //     
	alter_lpi.currentstatusprocess[indxnumberbattery_prm]=eeprom_read_byte(&Lpo_CurrentStatusprocess[indxnumberbattery_prm]); //    
		}

void LPI_ReadlionDeferred(bit indxnumberbattery_prm) //     Li-Ion 
	{
	alter_lpi.countercyclesalgorithm[indxnumberbattery_prm]=eeprom_read_byte(&Lin_CountcycleAlgorithm[indxnumberbattery_prm]); //     
	alter_lpi.currentstatusprocess[indxnumberbattery_prm]=eeprom_read_byte(&Lin_CurrentStatusprocess[indxnumberbattery_prm]); //    
	}

void LPI_ChargeParameters(bit indxnumberbattery_prm,word settcapacitybattery_prm) //   
	{
	byte algoritmdata_tmp=0; // 
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: algoritmdata_tmp=pgm_read_byte(&Lpi_ChargeBattery[alter_lpi.countercyclesalgorithm[FALSE]][0]); //   
					alter_lpi.limitfirstparameters[0]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    1
					if(alter_lpi.limitfirstparameters[0]>LPIMAXLIMIT_CURRENT) alter_lpi.limitfirstparameters[0]=LPIMAXLIMIT_CURRENT; //    1  2500  
					algoritmdata_tmp=pgm_read_byte(&Lpi_ChargeBattery[alter_lpi.countercyclesalgorithm[FALSE]][1]); //   
					if(algoritmdata_tmp>0) alter_lpi.limitfirstparameters[1]=(word)algoritmdata_tmp*100; //     1     
					else alter_lpi.limitfirstparameters[1]=LPIMAXBATT_VOLTAGE; //       
					algoritmdata_tmp=pgm_read_byte(&Lpi_ChargeBattery[alter_lpi.countercyclesalgorithm[FALSE]][2]); //   
					alter_lpi.limitfirstparameters[2]=(word)algoritmdata_tmp*10; //    40*  1
					algoritmdata_tmp=pgm_read_byte(&Lpi_ChargeBattery[alter_lpi.countercyclesalgorithm[FALSE]][3]); //   
					if(algoritmdata_tmp>0) //     
						{
						alter_lpi.limitfirstparameters[3]=settcapacitybattery_prm/(word)algoritmdata_tmp; //     
						if(alter_lpi.limitfirstparameters[3]<LPIMINLIMIT_CURRENT) alter_lpi.limitfirstparameters[3]=LPIMINLIMIT_CURRENT; //    10    
						}
					else alter_lpi.limitfirstparameters[3]=algoritmdata_tmp; //  
					break;
		case TRUE: algoritmdata_tmp=pgm_read_byte(&Lpi_ChargeBattery[alter_lpi.countercyclesalgorithm[TRUE]][0]); //   
					alter_lpi.limitsecondparameters[0]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    2
					if(alter_lpi.limitsecondparameters[0]>LPIMAXLIMIT_CURRENT) alter_lpi.limitsecondparameters[0]=LPIMAXLIMIT_CURRENT; //    2  2500  
					algoritmdata_tmp=pgm_read_byte(&Lpi_ChargeBattery[alter_lpi.countercyclesalgorithm[TRUE]][1]); //   
					if(algoritmdata_tmp>0) alter_lpi.limitsecondparameters[1]=(word)algoritmdata_tmp*100; //     2     
					else alter_lpi.limitsecondparameters[1]=LPIMAXBATT_VOLTAGE; //       
					algoritmdata_tmp=pgm_read_byte(&Lpi_ChargeBattery[alter_lpi.countercyclesalgorithm[TRUE]][2]); //   
					alter_lpi.limitsecondparameters[2]=(word)algoritmdata_tmp*10; //    40*  2 
					algoritmdata_tmp=pgm_read_byte(&Lpi_ChargeBattery[alter_lpi.countercyclesalgorithm[TRUE]][3]); //   
					if(algoritmdata_tmp>0) //     
						{
						alter_lpi.limitsecondparameters[3]=settcapacitybattery_prm/(word)algoritmdata_tmp; //     
						if(alter_lpi.limitsecondparameters[3]<LPIMINLIMIT_CURRENT) alter_lpi.limitsecondparameters[3]=LPIMINLIMIT_CURRENT; //    10    
						}
					else alter_lpi.limitsecondparameters[3]=algoritmdata_tmp; //  
		}
	}

void LPI_DischargeParameters(bit indxnumberbattery_prm,word settcapacitybattery_prm) //   
	{
	byte algoritmdata_tmp=0; // 
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: algoritmdata_tmp=pgm_read_byte(&Lpi_DischargeBattery[alter_lpi.countercyclesalgorithm[FALSE]][0]); //   
					alter_lpi.limitfirstparameters[0]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    1
					if(alter_lpi.limitfirstparameters[0]>LPIMAXLIMIT_CURRENT) alter_lpi.limitfirstparameters[0]=LPIMAXLIMIT_CURRENT; //     2500  
					algoritmdata_tmp=pgm_read_byte(&Lpi_DischargeBattery[alter_lpi.countercyclesalgorithm[FALSE]][1]); //   
					if(algoritmdata_tmp>0) alter_lpi.limitfirstparameters[1]=(word)algoritmdata_tmp*100; //     1   
					else alter_lpi.limitfirstparameters[1]=LPIMINBATT_VOLTAGE; //       
					algoritmdata_tmp=pgm_read_byte(&Lpi_DischargeBattery[alter_lpi.countercyclesalgorithm[FALSE]][2]); //   
					alter_lpi.limitfirstparameters[2]=(word)algoritmdata_tmp*10; //    40*  1 
					algoritmdata_tmp=pgm_read_byte(&Lpi_DischargeBattery[alter_lpi.countercyclesalgorithm[FALSE]][3]); //   
					if(algoritmdata_tmp>0) //     
						{
						alter_lpi.limitfirstparameters[3]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    
						if(alter_lpi.limitfirstparameters[3]<LPIMINLIMIT_CURRENT) alter_lpi.limitfirstparameters[3]=LPIMINLIMIT_CURRENT; //    10    
						}
					else alter_lpi.limitfirstparameters[3]=algoritmdata_tmp; //  
					break;
		case TRUE: algoritmdata_tmp=pgm_read_byte(&Lpi_DischargeBattery[alter_lpi.countercyclesalgorithm[TRUE]][0]); //   
					alter_lpi.limitsecondparameters[0]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    2
					if(alter_lpi.limitsecondparameters[0]>LPIMAXLIMIT_CURRENT) alter_lpi.limitsecondparameters[0]=LPIMAXLIMIT_CURRENT; //     2500   
					algoritmdata_tmp=pgm_read_byte(&Lpi_DischargeBattery[alter_lpi.countercyclesalgorithm[TRUE]][1]); //   
					if(algoritmdata_tmp>0) alter_lpi.limitsecondparameters[1]=(word)algoritmdata_tmp*100; //     2   
					else alter_lpi.limitsecondparameters[1]=LPIMINBATT_VOLTAGE; //       
					algoritmdata_tmp=pgm_read_byte(&Lpi_DischargeBattery[alter_lpi.countercyclesalgorithm[TRUE]][2]); //   
					alter_lpi.limitsecondparameters[2]=(word)algoritmdata_tmp*10; //    40*  2 
					algoritmdata_tmp=pgm_read_byte(&Lpi_DischargeBattery[alter_lpi.countercyclesalgorithm[TRUE]][3]); //   
					if(algoritmdata_tmp>0) //     
						{
						alter_lpi.limitsecondparameters[3]=settcapacitybattery_prm/(word)algoritmdata_tmp; //    
						if(alter_lpi.limitsecondparameters[3]<LPIMINLIMIT_CURRENT) alter_lpi.limitsecondparameters[3]=LPIMINLIMIT_CURRENT; //    10    
						}
					else alter_lpi.limitsecondparameters[3]=algoritmdata_tmp; //   
		}
	}

word LPI_ChargeController(bit indxnumberbattery_prm,word measuredcurrentbattery_prm,word measuredchargebattery_prm,word settingcurrentbattery_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm,word measuredtemperature_prm) //  
	{
	word settingcurrentbattery_tmp=0; //  
	word limitcurrentparameters_tmp=0; 
	word limitvoltageparameters_tmp=0; 
	settingcurrentbattery_tmp=settingcurrentbattery_prm; //      
	switch(indxnumberbattery_prm) //   
		{
		case FALSE: switch(alter_lpi.countercyclesalgorithm[FALSE]) //    1 
						{
						case 0:	if(measuredtemperature_prm>=alter_lpi.limitfirstparameters[2]) limitcurrentparameters_tmp=alter_lpi.limitfirstparameters[0]/2; //   0,25    40*   
								else limitcurrentparameters_tmp=alter_lpi.limitfirstparameters[0]; //   0,5     40* 
								limitvoltageparameters_tmp=alter_lpi.limitfirstparameters[1]; //      4190  
								settingcurrentbattery_tmp=LPI_ChargingCurrent(measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //   
								if(alter_lpi.currentstatusprocess[FALSE]!=2) alter_lpi.currentstatusprocess[FALSE]=2; // :   
								if(measuredvoltagebattery_prm>=alter_lpi.limitfirstparameters[1]) //   4190    
									{
									if(settingcurrentbattery_tmp>1) settingcurrentbattery_tmp/=2; //      
									alter_lpi.countercyclesalgorithm[FALSE]++; //        
									indx_lpi.updatingfirstdata=FALSE; //     
									}
								else if((measuredchargebattery_prm>4)&&(measuredvoltagebattery_prm<LPIMINBATT_VOLTAGE)) //   5      3000  
									{
									settingcurrentbattery_tmp=0; //         
									alter_lpi.countercyclesalgorithm[FALSE]=0; //   
									alter_lpi.currentstatusprocess[FALSE]=9; // :   
									indx_lpi.updatingfirstdata=FALSE; //      
									}
								break;
						case 1: limitcurrentparameters_tmp=alter_lpi.limitfirstparameters[0]; //    0,25  
								limitvoltageparameters_tmp=alter_lpi.limitfirstparameters[1]; //      4200  
								settingcurrentbattery_tmp=LPI_ChargingCurrent(measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lpi.currentstatusprocess[FALSE]!=3) alter_lpi.currentstatusprocess[FALSE]=3; // : 
								if((measuredcurrentbattery_prm<=alter_lpi.limitfirstparameters[3])&&(measuredvoltagebattery_prm>=alter_lpi.limitfirstparameters[1])) //            4200  
									{
									settingcurrentbattery_tmp=0; //         
									alter_lpi.countercyclesalgorithm[FALSE]=0; //   
									alter_lpi.currentstatusprocess[FALSE]=4; // :   
									indx_lpi.updatingfirstdata=FALSE; //     
									}
						}
					break;
		case TRUE: switch(alter_lpi.countercyclesalgorithm[TRUE]) //    2
						{
						case 0: if(measuredtemperature_prm>=alter_lpi.limitsecondparameters[2]) limitcurrentparameters_tmp=alter_lpi.limitsecondparameters[0]/2; //    0,25    40*  
								else limitcurrentparameters_tmp=alter_lpi.limitsecondparameters[0]; //    0,5     40*
								limitvoltageparameters_tmp=alter_lpi.limitsecondparameters[1]; //      4190  
								settingcurrentbattery_tmp=LPI_ChargingCurrent(measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lpi.currentstatusprocess[TRUE]!=2) alter_lpi.currentstatusprocess[TRUE]=2; // :  
								if(measuredvoltagebattery_prm>=alter_lpi.limitsecondparameters[1]) //   4190   
									{
									if(settingcurrentbattery_tmp>1) settingcurrentbattery_tmp/=2; //     
									alter_lpi.countercyclesalgorithm[TRUE]++; //      
									indx_lpi.updatingseconddata=FALSE; //     
									}
								else if((measuredchargebattery_prm>4)&&(measuredvoltagebattery_prm<LPIMINBATT_VOLTAGE)) //   5      3000 
									{
									settingcurrentbattery_tmp=0; //         
									alter_lpi.countercyclesalgorithm[TRUE]=0; //   
									alter_lpi.currentstatusprocess[TRUE]=9; // :   
									indx_lpi.updatingseconddata=FALSE; //     
									}
								break;
						case 1: limitcurrentparameters_tmp=alter_lpi.limitsecondparameters[0]; //    0,25 
								limitvoltageparameters_tmp=alter_lpi.limitsecondparameters[1]; //     4200  
								settingcurrentbattery_tmp=LPI_ChargingCurrent(measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lpi.currentstatusprocess[TRUE]!=3) alter_lpi.currentstatusprocess[TRUE]=3; // : 
								if((measuredcurrentbattery_prm<=alter_lpi.limitsecondparameters[3])&&(measuredvoltagebattery_prm>=alter_lpi.limitsecondparameters[1])) //            4200  
									{
									settingcurrentbattery_tmp=0; //         
									alter_lpi.countercyclesalgorithm[TRUE]=0; //   
									alter_lpi.currentstatusprocess[TRUE]=4; // :   
									indx_lpi.updatingseconddata=FALSE; //     
									}
						}
		}
	return settingcurrentbattery_tmp; //    
	}

word LPI_DischargeController(bit indxnumberbattery_prm,word measuredcurrentbattery_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm,word measuredchargebattery_prm,word settingcurrentbattery_prm,word measuredtemperature_prm) //  
	{
	word settingcurrentbattery_tmp=0; // 
	word limitcurrentparameters_tmp=0;
	word limitvoltageparameters_tmp=0;
	settingcurrentbattery_tmp=settingcurrentbattery_prm; //     
	switch(indxnumberbattery_prm) //  
		{
		case FALSE: switch(alter_lpi.countercyclesalgorithm[FALSE]) //    1
						{
						case 0: if(measuredtemperature_prm>=alter_lpi.limitfirstparameters[2]) limitcurrentparameters_tmp=alter_lpi.limitfirstparameters[0]/2; //    0,25    40*   
								else limitcurrentparameters_tmp=alter_lpi.limitfirstparameters[0]; //    0,5     40* 
								limitvoltageparameters_tmp=alter_lpi.limitfirstparameters[1]; //      3300  
								settingcurrentbattery_tmp=LPI_DischargingCurrent(measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lpi.currentstatusprocess[FALSE]!=6) alter_lpi.currentstatusprocess[FALSE]=6; // :  
								if(measuredvoltagebattery_prm<=alter_lpi.limitfirstparameters[1]) //      3300  
									{
									if(settingcurrentbattery_tmp>1) settingcurrentbattery_tmp/=2; //      
									alter_lpi.countercyclesalgorithm[FALSE]++; //       
									indx_lpi.updatingfirstdata=FALSE; //      
									}
								break;
						case 1: limitcurrentparameters_tmp=alter_lpi.limitfirstparameters[0]; //    0,2 
								limitvoltageparameters_tmp=alter_lpi.limitfirstparameters[1]; //     3000  
								settingcurrentbattery_tmp=LPI_DischargingCurrent(measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //   
								if(alter_lpi.currentstatusprocess[FALSE]!=7) alter_lpi.currentstatusprocess[FALSE]=7; // : 
								if((measuredcurrentbattery_prm<=alter_lpi.limitfirstparameters[3])&&(measuredvoltagebattery_prm<=alter_lpi.limitfirstparameters[1])) //            3000  
									{
									settingcurrentbattery_tmp=0; //         
									alter_lpi.countercyclesalgorithm[FALSE]=0; //   
									alter_lpi.currentstatusprocess[FALSE]=8; // :   
									indx_lpi.updatingfirstdata=FALSE; //     
									}
						}
					break;
		case TRUE: switch(alter_lpi.countercyclesalgorithm[TRUE]) //    2 
						{
						case 0: if(measuredtemperature_prm>=alter_lpi.limitsecondparameters[2]) limitcurrentparameters_tmp=alter_lpi.limitsecondparameters[0]/2; //    0,25    40*  
								else limitcurrentparameters_tmp=alter_lpi.limitsecondparameters[0]; //    0,5     40* 
								limitvoltageparameters_tmp=alter_lpi.limitsecondparameters[1]; //      3300  
								settingcurrentbattery_tmp=LPI_DischargingCurrent(measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lpi.currentstatusprocess[TRUE]!=6) alter_lpi.currentstatusprocess[TRUE]=6; // :  
								if(measuredvoltagebattery_prm<=alter_lpi.limitsecondparameters[1]) //      3300 
									{
									if(settingcurrentbattery_tmp>1) settingcurrentbattery_tmp/=2; //     
									alter_lpi.countercyclesalgorithm[TRUE]++; //      
									indx_lpi.updatingseconddata=FALSE; //     
									}
								break;
						case 1: limitcurrentparameters_tmp=alter_lpi.limitsecondparameters[0]; //    0,25 
								limitvoltageparameters_tmp=alter_lpi.limitsecondparameters[1]; //      3000  
								settingcurrentbattery_tmp=LPI_DischargingCurrent(measuredcurrentbattery_prm,settingcurrentbattery_prm,limitcurrentparameters_tmp,limitvoltageparameters_tmp,measuredvoltagebattery_prm,measvoltageloadedbattery_prm); //  
								if(alter_lpi.currentstatusprocess[TRUE]!=7) alter_lpi.currentstatusprocess[TRUE]=7; // : 
								if((measuredcurrentbattery_prm<=alter_lpi.limitsecondparameters[3])&&(measuredvoltagebattery_prm<=alter_lpi.limitsecondparameters[1])) //            3000  
									{
									settingcurrentbattery_tmp=0; //          
									alter_lpi.countercyclesalgorithm[TRUE]=0; //   
									alter_lpi.currentstatusprocess[TRUE]=8; // :   
									indx_lpi.updatingseconddata=FALSE; //     
									}
						}
		}
	return settingcurrentbattery_tmp; //     
	}

word LPI_ChargingCurrent(word measuredcurrentbattery_prm,word settingcurrentbattery_prm,word limitcurrentparameters_prm,word limitvoltageparameters_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm) //   
	{
	word settingcurrentbattery_tmp=0; // 
	settingcurrentbattery_tmp=settingcurrentbattery_prm; //     
	if((measuredcurrentbattery_prm<limitcurrentparameters_prm)&&(measuredvoltagebattery_prm<limitvoltageparameters_prm)&&(measvoltageloadedbattery_prm<LPIMAXLOAD_VOLTAGE)) settingcurrentbattery_tmp++; //             
	else if(((measuredcurrentbattery_prm>limitcurrentparameters_prm)||(measuredvoltagebattery_prm>limitvoltageparameters_prm)||(measvoltageloadedbattery_prm>=LPIMAXLOAD_VOLTAGE))&&(settingcurrentbattery_tmp>1)) settingcurrentbattery_tmp--; //             
	else asm("nop"); //    
	return settingcurrentbattery_tmp; //    
	}

word LPI_DischargingCurrent(word measuredcurrentbattery_prm,word settingcurrentbattery_prm,word limitcurrentparameters_prm,word limitvoltageparameters_prm,word measuredvoltagebattery_prm,word measvoltageloadedbattery_prm) //   
	{
	word settingcurrentbattery_tmp=0; // 
	settingcurrentbattery_tmp=settingcurrentbattery_prm; //     
	if((measuredcurrentbattery_prm<limitcurrentparameters_prm)&&(measuredvoltagebattery_prm>limitvoltageparameters_prm)&&(measvoltageloadedbattery_prm>LPIMINLOAD_VOLTAGE)) settingcurrentbattery_tmp++; //             
	else
		{
		if((limitvoltageparameters_prm==3000)&&(measuredvoltagebattery_prm<limitvoltageparameters_prm)&&(settingcurrentbattery_tmp>1)) settingcurrentbattery_tmp/=2; //        3000  
		else if(((measuredcurrentbattery_prm>limitcurrentparameters_prm)||(measuredvoltagebattery_prm<limitvoltageparameters_prm)||(measvoltageloadedbattery_prm<=LPIMINLOAD_VOLTAGE))&&(settingcurrentbattery_tmp>1)) settingcurrentbattery_tmp--; //              3300  
		else asm("nop"); //    
		}
	return settingcurrentbattery_tmp; //   
	}

byte LPI_ReadcurrentStatus(bit indxnumberbattery_prm) //   
	{
	byte currentstatusprocess_tmp=0; //  
	currentstatusprocess_tmp=alter_lpi.currentstatusprocess[indxnumberbattery_prm]; //    
	return currentstatusprocess_tmp; //   
	}













